@@ -16,6 +16,7 @@ dsb = string_magic_alias(.delim = ".[ ]")
 # collapse ####
 #
 
+
 # regular
 x = 1:3
 test(string_magic("win = {'-'c ? x}"), "win = 1-2-3")
@@ -42,6 +43,7 @@ test(txt, c("bon.les.gen", "com.ca.va?", "bie", "bie"))
 # split ####
 #
 
+
 x = "Cogito...Ergo...Sum"
 txt = string_magic("He said: {'f/...'s ? x}")
 test(txt, c("He said: Cogito", "He said: Ergo", "He said: Sum"))
@@ -55,6 +57,7 @@ test(txt, c("He said: ", "He said: ogito...Ergo...", "He said: um"))
 # replace ####
 #
 
+
 x = c("Blanche dit 'Bla bla blanc'.")
 txt = string_magic("She said: {'bla'r, ws ? x}")
 test(txt, "She said: Blanche dit 'Bla nc'.")
@@ -131,6 +134,7 @@ test(txt, c("Stare at the []", "Stare at the Poon", "Stare at the Pun"))
 # x: single extraction ####
 #
 
+
 x = c("Blanche dit", " 'Bla bla blanc'.")
 txt = string_magic("mots: {'bla'x ? x}")
 test(txt, c("mots: ", "mots: bla"))
@@ -160,6 +164,7 @@ test(string_magic("mots: {'iw/bla'extract.first ? x}"),
 # X: multiple extractions ####
 #
 
+
 x = c("Blanche dit", " 'Bla bla blanc'.")
 txt = string_magic("{'bla'X ? x}")
 test(txt, c("bla", "bla"))
@@ -185,6 +190,7 @@ test(txt, "info: 57-26, 32-7, 29-8 and 55")
 # is, get, which ####
 #
 
+
 x = c("Hi Mary.", "Hi Charles!", "Are you OK?", "I think so Charles.", "Great to hear, Mary!")
 txt = string_magic("A few quotes:\n{Q, 'mary | charles & \\!'get.ignore, '  'paste, '\n'c ? x}")
 test(txt, "A few quotes:\n  \"Hi Charles!\"\n  \"Great to hear, Mary!\"")
@@ -209,6 +215,7 @@ test(txt, "info: laura-and, charles-and, charly-and and june")
 # each/times ####
 #
 
+
 test(string_magic("I like {5 times.c ! ?} marks!"), "I like ????? marks!")
 
 x = c("mary", "richard")
@@ -222,6 +229,7 @@ test(string_magic("values: {2 each.c ? c('a', 'b')}"), "values: aabb")
 # case ####
 #
 
+
 x = "where is bryan? Bryan is in the KITCHEN."
 
 txt = dsb(".[lower ? x]")
@@ -253,6 +261,7 @@ test(txt, "Results From a New Estimator: A New Hope")
 # quotes ####
 #
 
+
 x = "siren"
 txt = string_magic("Is it the song of the {q ? x}, of the {Q ? x} or of the {bq ? x}?")
 test(txt, "Is it the song of the 'siren', of the \"siren\" or of the `siren`?")
@@ -261,6 +270,7 @@ test(txt, "Is it the song of the 'siren', of the \"siren\" or of the `siren`?")
 # format ####
 #
 
+
 x = c(1, 123, 123456)
 txt = string_magic("The numbers are:\n{'\n'c ! - {format ? x} | {rev, Format ? x}}")
 test(txt, "The numbers are:\n- 1       | 123,456\n- 123     |     123\n- 123,456 |       1")
@@ -285,6 +295,7 @@ test_contains(txt, "\\d:\\d & \\d{4}")
 # sprintf ####
 #
 
+
 txt = string_magic("pi = {%.3f ? pi}")
 test(txt, "pi = 3.142")
 
@@ -302,6 +313,7 @@ test(txt, "The winners are:\n- michael   !\n- ana       !")
 # ws ####
 #
 
+
 x = " &Hej! Welcome!! A\t\n 6-feet wide bed awaits!\t."
 
 txt = dsb(".[ws ? x]")
@@ -332,6 +344,7 @@ test(txt, "Hej Welcome feet wide bed awaits")
 # tws ####
 #
 
+
 x = c("  bonjour les gens", "ahem \n  ", NA)
 txt = dsb(".[tws ? x]")
 test(txt, c("bonjour les gens", "ahem", NA))
@@ -340,6 +353,7 @@ test(txt, c("bonjour les gens", "ahem", NA))
 # trim ####
 #
 
+
 x = c("bonjour", "ahem")
 txt = dsb(".[4 trim ? x]")
 test(txt, c("our", ""))
@@ -357,6 +371,7 @@ test(txt, c("onjou", "he"))
 # k, keep char ####
 #
 
+
 x = c("this is a long sentence", "a short one")
 txt = string_magic("{11k, ' and 'c ? x}")
 test(txt, "this is a l and a short one")
@@ -386,6 +401,7 @@ test(txt, "this is a .. and a short one")
 # K, keep elements ####
 #
 
+
 x = 1:5
 
 txt = string_magic("{3K ? x}")
@@ -417,10 +433,14 @@ test(string_magic("info: {' 'S, ~(2Ko), C ? x}"), "err")
 # enum ####
 #
 
+
 x = 1:5
 txt = string_magic("{enum.q.3.oxf ? x}")
 test(txt, "'1', '2', and 3 others")
 
+txt = string_magic("{enum.q.3.comm ? x}")
+test(txt, "'1', '2', 3 others")
+
 txt = string_magic("{enum.bq.or ! x{1:3}}")
 test(txt, "`x1`, `x2` or `x3`")
 
@@ -442,6 +462,7 @@ test(txt, "1, 2 and 3 ; a, b and c ; a, 1, b and 2")
 # first and last ###
 #
 
+
 x = 1:5
 txt = string_magic("{2 first, ''c ? x}")
 test(txt, "12")
@@ -471,6 +492,7 @@ test(txt, "3 ; c ; 2")
 # firstchar, lastchar ####
 #
 
+
 x = c("bonjour", "les", "gens")
 a = 2
 txt = string_magic("{3 firstchar, `a`lastchar ? x}")
@@ -480,6 +502,7 @@ test(txt, c("on", "es", "en"))
 # rev ####
 #
 
+
 test(string_magic("{rev ? 1:3}"), 3:1)
 
 # conditional
@@ -491,6 +514,7 @@ test(txt, "321 ; cba ; 2b1a")
 # sort, dsort ####
 #
 
+
 x = c(5, 3, 8, 1)
 test(string_magic("{sort ? x}"), c(1, 3, 5, 8))
 
@@ -516,6 +540,7 @@ test(txt, "521 ; ecba")
 # paste ####
 #
 
+
 x = 1:2
 txt = string_magic("{'x'paste, ', 'c ? x}")
 test(txt, "x1, x2")
@@ -538,6 +563,7 @@ test(txt, "The number is 5.")
 # insert ####
 #
 
+
 x = 1:2
 txt = string_magic("{'0'insert, ''c ? x}")
 test(txt, "012")
@@ -556,6 +582,7 @@ test(string_magic("{''S, ~('0'insert, ''c), ' ; 'c ? x}"), "err")
 # fill ####
 #
 
+
 x = c("bon", "bonjour les gens")
 txt = string_magic("{fill, q, C ? x}")
 test(txt, "'bon             ' and 'bonjour les gens'")
@@ -564,7 +591,7 @@ txt = string_magic("{fill.right, q, C ? x}")
 test(txt, "'             bon' and 'bonjour les gens'")
 
 txt = string_magic("{fill.center, q, C ? x}")
-test(txt, "'      bon       ' and 'bonjour les gens'")
+test(txt, "'       bon      ' and 'bonjour les gens'")
 
 x = c(5, 15)
 txt = string_magic("{3 fill.right, q, C ? x}")
@@ -580,6 +607,7 @@ test(txt, "'005' and '015'")
 # join ####
 #
 
+
 x = "\n    bonjour \\\n   les gens\n est-ce que ca va?"
 txt = string_magic("Text: {tws, join ? x}")
 test(txt, "Text: bonjour les gens\n est-ce que ca va?")
@@ -588,6 +616,7 @@ test(txt, "Text: bonjour les gens\n est-ce que ca va?")
 # escape ####
 #
 
+
 x = "bon\njour \t les \t gens"
 txt = string_magic(x, .last = "escape")
 test(txt, "bon\\njour \\t les \\t gens")
@@ -596,6 +625,7 @@ test(txt, "bon\\njour \\t les \\t gens")
 # unik ####
 #
 
+
 x = c(1, 1, 2, 3, 3, 3) 
 test(string_magic("{unik ? x}"), 1:3)
 
@@ -604,10 +634,60 @@ x = c("11211125564454", "aggafsgaffasg")
 txt = string_magic("{''S, ~(unik, ''c), ' ; 'c ? x}")
 test(txt, "12564 ; agfs")
 
+#
+# table ####
+#
+
+x = strsplit("aggafsggaffasg", "")[[1]]
+
+# default
+txt = sma("letters: {table, enum ? x}.")
+test(txt, "letters: g (5), a (4), f (3) and s (2).")
+
+txt = sma("letters: {table.sort, enum ? x}.")
+test(txt, "letters: a (4), f (3), g (5) and s (2).")
+
+txt = sma("letters: {table.dsort, enum ? x}.")
+test(txt, "letters: s (2), g (5), f (3) and a (4).")
+
+txt = sma("letters: {table.fsort, enum ? x}.")
+test(txt, "letters: s (2), f (3), a (4) and g (5).")
+
+txt = sma("letters: {table.nosort, enum ? x}.")
+test(txt, "letters: a (4), g (5), f (3) and s (2).")
+
+# with argument
+txt = sma("letters: {'{q ? x} {round(s * 100)}%'table, enum ? x}.")
+test(txt, "letters: 'g' 36%, 'a' 29%, 'f' 21% and 's' 14%.")
+
+txt = sma("letters: {'{x}: {n}'table.sort, ', 'c ? x}.")
+test(txt, "letters: a: 4, f: 3, g: 5, s: 2.")
+
+# error
+test(sma("letters: {'x = n'table, ', 'c ? x}."), "no interpolated", "err")
+
+test(sma("letters: {'{zz}'table, ', 'c ? x}."), "following invalid", "err")
+
+# double sort
+x = sample(stsplit("aacceebbbuuu", ""))
+
+txt = string_ops(x, "table.dfsort.sort, ', 'c")
+test(txt, "b (3), u (3), a (2), c (2), e (2)")
+
+txt = string_ops(x, "table.fsort.sort, ', 'c")
+test(txt, "a (2), c (2), e (2), b (3), u (3)")
+
+txt = string_ops(x, "table.dfsort.dsort, ', 'c")
+test(txt, "u (3), b (3), e (2), c (2), a (2)")
+
+txt = string_ops(x, "table.fsort.dsort, ', 'c")
+test(txt, "e (2), c (2), a (2), u (3), b (3)")
+
 #
 # nth ####
 #
 
+
 x = c(1, 6)
 txt = string_magic("They arrived {nth, C ? x}.")
 test(txt, "They arrived 1st and 6th.")
@@ -619,6 +699,7 @@ test(txt, "They arrived first and sixth.")
 # ntimes ####
 #
 
+
 x = c(1, 6)
 txt = string_magic("They won {ntimes, C ? x}.")
 test(txt, "They won once and 6 times.")
@@ -630,6 +711,7 @@ test(txt, "They won once and six times.")
 # n ####
 #
 
+
 x = c(45546, "bonjour")
 txt = string_magic("A = {n ? x} ; B = {n.l ? 55}")
 test(txt, c("A = 45,546 ; B = fifty-five", "A = bonjour ; B = fifty-five"))
@@ -648,6 +730,7 @@ test(txt, "This test was written in mmxxiii.")
 # len ####
 #
 
+
 x = 1:5
 txt = string_magic("{Len.up ? x} numbers.")
 test(txt, "Five numbers.")
@@ -672,14 +755,16 @@ test(num, c(3, 2, 4))
 # width ####
 #
 
+
 x = "Rome, l'unique objet de mon ressentiment, Rome a qui vient ton bras d'immoler mon amant"
-txt = string_magic("Voici le texte a apprendre:\n{'40|>'width ? x}.")
+txt = string_magic("Voici le texte a apprendre:\n{'40|>'swidth ? x}.")
 test(txt, c("Voici le texte a apprendre:\n> Rome, l'unique objet de mon\n> ressentiment, Rome a qui vient ton\n> bras d'immoler mon amant."))
 
 #
 # difftime ####
 #
 
+
 x = 3654
 txt = string_magic("Time since last check: {difftime ? x}.")
 test(txt, "Time since last check: 1 hour 00 min.")
@@ -692,6 +777,7 @@ test(txt, "Time since last check: 41 min 42 sec.")
 # erase ####
 #
 
+
 x = 1:5
 txt = string_magic("{if(. <= 3 ; erase), '.'c ? x}")
 test(txt, "...4.5")
@@ -700,6 +786,7 @@ test(txt, "...4.5")
 # nuke ####
 #
 
+
 x = 1:5
 txt = string_magic("nothing = {nuke ? x}")
 test(txt, "nothing = ")
@@ -708,6 +795,7 @@ test(txt, "nothing = ")
 # rm ####
 #
 
+
 x = c("", "    ", "556", ":!", "pour qui sont ces", "serpents qui sifflent sur nos tetes?")
 txt = string_magic("{5 firstchar, rm, ', 'c ? x}")
 test(txt, "    , 556, :!, pour , serpe")
@@ -736,6 +824,7 @@ test(txt, "laura-and, charles-and, charly-and and june")
 # num ####
 #
 
+
 x = "parse55this"
 txt = string_ops(x, "'\\d+'x, num")
 test(txt, 55)
@@ -768,6 +857,7 @@ test(txt, "57-26, 32-7, 29-8 and 55")
 # stop ####
 #
 
+
 x = "Hi I'm Laurent and I'm trying to remove stop-words."
 txt = string_magic("Before: {x}\nAfter: {stop, ws ? x}")
 test(txt, "Before: Hi I'm Laurent and I'm trying to remove stop-words.\nAfter: Hi Laurent trying remove stop-words.")
@@ -776,6 +866,7 @@ test(txt, "Before: Hi I'm Laurent and I'm trying to remove stop-words.\nAfter: H
 # if-else ####
 #
 
+
 # elementwise
 x = c(15, 550)
 txt = dsb("The value is .[& x > 50 ; > 50 ; <= 50]")
@@ -803,7 +894,11 @@ test(txt, "The max value is .")
 
 # nesting
 x = c(15, 550)
-txt = string_magic("The values are{& length(x) < 5 ; ': {C ? x}' ; 'too many'}.")
+txt = string_magic("The values are{& length(x) < 5 ; ': {C ? .}' ; ' too many'}.")
+test(txt, "The values are: 15 and 550.")
+
+data = data.frame(x = x, stringsAsFactors = FALSE)
+txt = string_magic("The values are{& length(data$x) < 5 ; ': {C ? .}' ; ' too many'}.")
 test(txt, "The values are: 15 and 550.")
 
 # special if else
@@ -818,13 +913,14 @@ txt = string_magic("{C ! {&&x %% 2 == 1;{y}}}")
 test(txt, "a, 2 and c")
 
 z = tail(letters, 3)
-txt = string_magic("{C ! {&x %% 2 == 1; {y} ; {bq?z}}}")
-test(txt, "`x`,  b and `z`")
+txt = string_magic("{C ! {&x %% 2 == 1 ; {y} ; {bq?z}}}")
+test(txt, "a, `y` and c")
 
 #
 # if ####
 #
 
+
 # special values: ., .len (or .N), .nchar (or .C)
 
 x = c(5, 25, 30, 7)
@@ -856,6 +952,7 @@ test(txt, "Example: x5, 2/5 and 3/0.")
 # vif ####
 #
 
+
 x = c(5, 25, 30, 7)
 txt = string_magic("There are {vif(.len<3 ; not many ; many), C ? x} numbers.")
 test(txt, "There are many numbers.")
@@ -887,6 +984,7 @@ test(txt, "y = 5 + 12 + 20")
 # escaping ####
 #
 
+
 txt = string_magic("\\{} it's some braces")
 test(txt, "{} it's some braces")
 
@@ -903,6 +1001,7 @@ test(txt, "I'm saying 'ha ] ha'!")
 # user-defined ops ####
 #
 
+
 # without scope
 xplode = function(x, argument, options, ...){
   unlist(strsplit(as.character(x), ""))
@@ -965,6 +1064,7 @@ test_err_contains(string_vec("hi, {there"), "bracket & matched & escape")
 # multi line expressions ####
 #
 
+
 txt = string_magic("The solution to x + 32 = 5 is {
      y = 32
      z = 5
@@ -985,6 +1085,7 @@ test(txt, "First letters: abcdef")
 # variables as arguments ####
 #
 
+
 txt = string_magic("Hi {you}!", you = "Omer")
 test(txt, "Hi Omer!")
 
@@ -1013,6 +1114,7 @@ test(txt, "The number is error")
 # default options ####
 #
 
+
 sma2 = string_magic_alias(.delim = "$[ ]")
 txt = sma2("x$[1:2]", .last = "C")
 test(txt, "x1 and x2")
@@ -1021,6 +1123,7 @@ test(txt, "x1 and x2")
 # .post + .last ####
 #
 
+
 x = string_magic("{1:5}0", .last = "num", .post = sum)
 test(x, 150)
 
@@ -1028,6 +1131,7 @@ test(x, 150)
 # .default ####
 #
 
+
 x = string_magic("{1:5}0", .default = "'1|1'first, enum")
 test(x, "1 and 50")
 
